using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading;
using CODE.Framework.Wpf.Mvvm;

namespace MagazineManagement.Models.Subscriber
{
    public class ListViewModel : ViewModel
    {
        public ListViewModel()
        {
            Subscribers = new ObservableCollection<StandardViewModel>();

            Actions.Add(new ViewAction("Print", brushResourceKey: "CODE.Framework-Icon-Print", execute: (a, o) => Controller.Message("The print example has not been implemented.", "Example"), significance: ViewActionSignificance.Highest));
            Actions.Add(new CloseCurrentViewAction(this, beginGroup: true) {Significance = ViewActionSignificance.Highest});

            Search = new ViewAction("Search", execute: (o, a) => LoadData());
            EditSubscriber = new ViewAction("Edit", execute: (o, a) => Controller.Action("Subscriber", "Edit"));
        }

        private void LoadData()
        {
            AsyncWorker.Execute(() =>
                {
                    var result = new List<StandardViewModel>();
                    for (var counter = 1; counter <= 100; counter++)
                    {
                        var subscriber = new StandardViewModel
                            {
                                Text1 = GetRandomFirstName() + " " + GetRandomLastName(),
                                Text2 = GetRandomCompany()
                            };
                        result.Add(subscriber);
                    }
                    Thread.Sleep(5000); // Simulating a slow process
                    return result;
                }, result =>
                    {
                        Subscribers.Clear();
                        foreach (var subscriber in result)
                        {
                            subscriber.LoadSharedImage1FromBrushResource("CODE.Framework-Icon-ContactInfo");
                            Subscribers.Add(subscriber);
                        }
                    }, this);
        }

        private static string GetRandomFirstName()
        {
            return GetRandomString(new List<string> {"John", "James", "Peter", "David", "Steve", "Kimberly", "Susan", "Ellen"});
        }
        private static string GetRandomLastName()
        {
            return GetRandomString(new List<string> { "Smith", "Jones", "Doe", "Gates", "Mayer", "MacLeod" });
        }
        private static string GetRandomCompany()
        {
            return GetRandomString(new List<string> { "Microsoft", "Apple", "Google", "EPS", "CODE", "Tower 48" });
        }
        private static string GetRandomString(ICollection<string> list)
        {
            var rand = new Random(Seed);
            var skip = rand.Next(list.Count - 1);
            return list.Skip(skip).FirstOrDefault();
        }

        private static int _seed = -1;
        private static int Seed
        {
            get 
            { 
                if (_seed == -1) _seed = Environment.TickCount;
                _seed++;
                return _seed;
            }
        }

        public IViewAction Search { get; set; }
        public IViewAction EditSubscriber { get; set; }

        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string Company { get; set; }

        public ObservableCollection<StandardViewModel> Subscribers { get; set; }
    }
}
